enum Planet {Merkur, Venus, Erde, Mars, Jupiter, Saturn, Uranus, Neptun};

double rads = M_PI / 180.0;

struct Vec3 {
	double x,y,z;
};

double mod2pi(double x) {   
	double b = x / (2*M_PI);
	double a = (2*M_PI)*(b - int(b));
	if (a < 0) {
		a = (2*M_PI) + a;
	}
	return a;
}

struct Bahnelemente {
	double a;
	double eps;
	double i;
	double Omega;
	double pii;
	double L;
	
	double meanAnomaly() {	
		return mod2pi(L - pii);
	}
		
	double excentricAnomaly(double M) {
		// iterative Lösung der Kepler-Gleichung

		double epsilon = 1e-12;
		double E0 = M + eps * sin(M) + (1*eps*cos(M));

		for (int i=0;i<10;i++) {
			double E1 = E0;
			E0 = E1 - (E1-eps*sin(E1) -M)/(1-eps*cos(E1));
			if (abs(E0-E1)<epsilon) {
				break;
			}
		}		
		return E0;
	}
	
	double trueAnomaly(double E) {
		double phi = 2*atan(sqrt((1+eps)/(1-eps))*tan(.5*E));
		return mod2pi(phi);
	}

	Vec3 cartCoord(double phi) {
		double r = a * (1-eps*eps) /(1+eps*cos(phi));
		
		Vec3 pos;
		pos.x = r*(cos(Omega)*cos(phi + pii - Omega) - sin(Omega)*sin(phi+pii-Omega)*cos(i));
		pos.y = r*(sin(Omega)*cos(phi + pii - Omega) + cos(Omega)*sin(phi+pii-Omega)*cos(i));
		pos.z = r*sin(phi + pii - Omega) * sin(i);
		return pos;
	}

	Bahnelemente(Planet p, double T) {
		// p ist Planet
		// T ist Zeit seit J2000.0 in Julianischen Jahrhunderten
		// a in AU
		// Winkel in Bogenmass

		if (p==Merkur) { 
			a =        0.38709893 +         0.00000066*T;
			eps =      0.20563069 +         0.00002527*T;
			i =         (7.00487  -        23.51*T/3600)*rads;
			Omega =    (48.33167  -       446.30*T/3600)*rads;
			pii =      (77.45645  +       573.57*T/3600)*rads;
			L = mod2pi((252.25084 + 538101628.29*T/3600)*rads);
		}

		if (p==Venus) { 
			a =        0.72333199 +         0.00000092*T;
			eps =      0.00677323 -         0.00004938*T;
			i =          (3.39471 -         2.86*T/3600)*rads;
			Omega =     (76.68069 -       996.89*T/3600)*rads;
			pii =      (131.53298 -       108.80*T/3600)*rads;
			L = mod2pi((181.97973 + 210664136.06*T/3600)*rads);
		}

		if (p==Erde) {
			a =           1.00000011 -      0.00000005*T;
			eps =         0.01671022 -      0.00003804*T;
			i =          (0.00005 -        46.94*T/3600)*rads;
			Omega =    (-11.26064 -     18228.25*T/3600)*rads;
			pii =      (102.94719 +      1198.28*T/3600)*rads;
			L = mod2pi((100.46435 + 129597740.63*T/3600)*rads);
		}

		if (p==Mars) { 
			a =        1.52366231 -        0.00007221*T;
			eps =      0.09341233 +        0.00011902*T;
			i =          (1.85061 -       25.47*T/3600)*rads;
			Omega =     (49.57854 -     1020.19*T/3600)*rads;
			pii =      (336.04084 +     1560.78*T/3600)*rads;
			L = mod2pi((355.45332 + 68905103.78*T/3600)*rads);
		}

		if (p==Jupiter) {
			a =           5.20336301 + 0.00060737*T;
			eps =         0.04839266 - 0.00012880*T;
			i =          (1.30530 -    4.15*T/3600)*rads;
			Omega =    (100.55615 + 1217.17*T/3600)*rads;
			pii =       (14.75385 +  839.93*T/3600)*rads;
			L =  mod2pi((34.40438 + 10925078.35*T/3600)*rads);
		}
		
		if (p==Saturn) {
			a =       9.53707032 -       0.00301530*T;
			eps =     0.05415060 -       0.00036762*T;
			i =         (2.48446 +       6.11*T/3600)*rads;
			Omega =   (113.71504 -    1591.05*T/3600)*rads;
			pii =      (92.43194 -    1948.89*T/3600)*rads;
			L = mod2pi((49.94432 + 4401052.95*T/3600)*rads);
		}

		if (p==Uranus) {
			a = 19.19126393 + 0.00152025*T;
			eps =  0.04716771 - 0.00019150*T;
			i = (  0.76986  -    2.09*T/3600)*rads;
			Omega = ( 74.22988  - 1681.40*T/3600)*rads;
			pii = (170.96424  + 1312.56*T/3600)*rads;
			L = mod2pi((313.23218 + 1542547.79*T/3600)*rads);
		}

		if (p==Neptun) {
			a = 30.06896348 - 0.00125196*T;
			eps =  0.00858587 + 0.00002510*T;
			i = (  1.76917  -   3.64*T/3600)*rads;
			Omega = (131.72169  - 151.25*T/3600)*rads;
			pii = ( 44.97135  - 844.43*T/3600)*rads;
			L = mod2pi((304.88003 + 786449.21*T/3600)*rads);
		}
	}
};
